﻿using SelfPropelledArtilleryUnit.CollectionGenericObjects;
using SelfPropelledArtilleryUnit.Drawnings;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SelfPropelledArtilleryUnit;

public partial class FormPropelledArtilleryCollection : Form
{
    /// <summary>
	/// Хранилише коллекций
	/// </summary>
	private readonly StorageCollection<DrawningPropelledArtillery> _storageCollection;

    /// <summary>
	/// Компания
	/// </summary>
	private AbstractCompany? _company = null;

    /// <summary>
	/// Конструктор
	/// </summary>
    public FormPropelledArtilleryCollection()
    {
        InitializeComponent();
        _storageCollection = new();
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }
    private void buttonAddPropelledArtillery_Click_1(object sender, EventArgs e) => CreateObject(nameof(DrawningPropelledArtillery));


    private void buttonAddSelfPropelledArtilleryUnit_Click_1(object sender, EventArgs e) => CreateObject(nameof(DrawningSelfPropelledArtilleryUnit));

    //   /// <summary>
    ///// Добавление Бронированная машина
    ///// </summary>
    ///// <param name="sender"></param>
    ///// <param name="e"></param>
    //private void ButtonAddPropelledArtillery_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningPropelledArtillery));

    //   /// <summary>
    //   /// Добавление Самоходная арт. установка
    //   /// </summary>
    //   /// <param name="sender"></param>
    //   /// <param name="e"></param>
    //   private void ButtonAddSelfPropelledArtilleryUnit_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningSelfPropelledArtilleryUnit));

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }
        Random random = new();
        DrawningPropelledArtillery drawningPropelledArtillery;
        switch (type)
        {
            case nameof(DrawningPropelledArtillery):
                drawningPropelledArtillery = new DrawningPropelledArtillery(random.Next(100, 300),
                random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningSelfPropelledArtilleryUnit):
                // TODO вызов диалогового окна для выбора цвета (made)
                drawningPropelledArtillery = new DrawningSelfPropelledArtilleryUnit(random.Next(100, 300), random.Next(1000, 3000),
                GetColor(random), GetColor(random),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }
        if (_company + drawningPropelledArtillery != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
	/// Получение цвета
	/// </summary>
	/// <param name="random">Генератор случайных чисел</param>
	/// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    private void maskedTextBox_MaskInputRejected(object sender, MaskInputRejectedEventArgs e)
    {

    }

    /// <summary>
	/// Удаление объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void buttonRemovePropelledArtillery_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBox.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningPropelledArtillery? propelledartillery = null;
        int counter = 100;
        while (propelledartillery == null)
        {
            propelledartillery = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        if (propelledartillery == null)
        {
            return;
        }

        SelfPropelledArtilleryUnit form = new()
        {
            SetPropelledArtillery = propelledartillery
        };
        form.ShowDialog();
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }

    private void buttonColectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }
        _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
        RerfreshListBoxItems();
    }
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }
    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        // TODO прописать логику удаления элемента из коллекции
        // нужно убедиться, что есть выбранная коллекция
        // спросить у пользователя через MessageBox, что он подтверждает, что хочет удалить запись
        // удалить и обновить ListBox
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        RerfreshListBoxItems();
    }

    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<DrawningPropelledArtillery>? collection =
            _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new ArtilleryBase(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }
}


